//!
//!
use crate::repositories::member::models::Member as _Member;
use crate::repositories::member_bank_deposit::schema::MemberBankDepositState;
use diesel::deserialize::QueryableByName;

///
#[derive(PartialEq, Debug, Clone)]
pub struct MemberBankDepositModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub member: _Member,
  ///
  pub name: String,
  ///
  pub amount: f64,
  ///
  pub memo: Option<String>,
  ///
  pub state: MemberBankDepositState,
  ///
  pub state_changed_at: i64,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}

impl QueryableByName<diesel::pg::Pg> for MemberBankDepositModel {
  fn build<R: diesel::row::NamedRow<diesel::pg::Pg>>(row: &R) -> diesel::deserialize::Result<Self> {
    let member = _Member {
      id: row.get("_m_id")?,
      site_id: row.get("_m_site_id")?,
      member_class_id: row.get("_m_member_class_id")?,
      member_level_id: row.get("_m_member_level_id")?,
      username: row.get("_m_username")?,
      password: row.get("_m_password")?,
      nickname: row.get("_m_nickname")?,
      mobile_phone_number: row.get("_m_mobile_phone_number")?,
      state: row.get("_m_state")?,
      state_changed_at: row.get("_m_state_changed_at")?,
      parent_member_id: row.get("_m_parent_member_id")?,
      child_member_count: row.get("_m_child_member_count")?,
      last_signined_ip: row.get("_m_last_signined_ip")?,
      last_signined_at: row.get("_m_last_signined_at")?,
      created_at: row.get("_m_created_at")?,
      updated_at: row.get("_m_updated_at")?,
      deleted_at: row.get("_m_deleted_at")?,
    };

    Ok(MemberBankDepositModel {
      id: row.get("mbd_id")?,
      member,
      name: row.get("mbd_name")?,
      amount: row.get("mbd_amount")?,
      memo: row.get("mbd_memo")?,
      state: row.get("mbd_state")?,
      state_changed_at: row.get("mbd_state_changed_at")?,
      created_at: row.get("mbd_created_at")?,
      updated_at: row.get("mbd_updated_at")?,
    })
  }
}
